@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004,2006 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -22,6 +23,7 @@ import org.apache.xerces.impl.msg.XMLMessageFormatter;
 import org.apache.xerces.util.SymbolTable;
 import org.apache.xerces.util.XMLChar;
 import org.apache.xerces.util.XMLStringBuffer;
+import org.apache.xerces.util.XMLSymbols;
 import org.apache.xerces.xni.Augmentations;
 import org.apache.xerces.xni.XMLDTDContentModelHandler;
 import org.apache.xerces.xni.XMLDTDHandler;
@@ -57,7 +59,7 @@ import org.apache.xerces.xni.parser.XMLInputSource;
  * @author Glenn Marcy, IBM
  * @author Eric Ye, IBM
  *
- * @version $Id: XMLDTDScannerImpl.java 377620 2006-02-14 04:56:13Z mrglavas $
+ * @version $Id: XMLDTDScannerImpl.java 454439 2006-10-09 17:45:39Z mrglavas $
  */
 public class XMLDTDScannerImpl
     extends XMLScanner
@@ -296,10 +298,10 @@ public class XMLDTDScannerImpl
      * @param standalone True if the document was specified as standalone.
      *                   This value is important for verifying certain
      *                   well-formedness constraints.
-     * @param hasExternalDTD True if the document has an external DTD.
-     *                       This allows the scanner to properly notify
-     *                       the handler of the end of the DTD in the
-     *                       absence of an external subset.
+     * @param hasExternalSubset True if the document has an external DTD.
+     *                          This allows the scanner to properly notify
+     *                          the handler of the end of the DTD in the
+     *                          absence of an external subset.
      *
      * @return True if there is more to scan, false otherwise.
      */
@@ -647,12 +649,9 @@ public class XMLDTDScannerImpl
     }
 
     /** 
-     * Dispatch an XML "event".
+     * Dispatch an XML "event".              
      *
-     * @param complete True if this method is intended to scan
-     *                 and dispatch as much as possible.                 
-     *
-     * @return True if a TextDecl was scanned.
+     * @return true if a TextDecl was scanned.
      *
      * @throws IOException  Thrown on i/o error.
      * @throws XNIException Thrown on parse error.
@@ -1266,7 +1265,12 @@ public class XMLDTDScannerImpl
                 String aName = fEntityScanner.scanName();
                 if (aName == null) {
                     reportFatalError("MSG_NAME_REQUIRED_IN_NOTATIONTYPE",
-                                     new Object[]{elName, atName}); 
+                                     new Object[]{elName, atName});
+                    c = skipInvalidEnumerationValue();
+                    if (c == '|') {
+                        continue;
+                    }
+                    break;
                 }
                 ensureEnumerationSize(fEnumerationCount + 1);
                 fEnumeration[fEnumerationCount++] = aName;
@@ -1295,6 +1299,11 @@ public class XMLDTDScannerImpl
                 if (token == null) {
                     reportFatalError("MSG_NMTOKEN_REQUIRED_IN_ENUMERATION",
                                      new Object[]{elName, atName});
+                    c = skipInvalidEnumerationValue();
+                    if (c == '|') {
+                        continue;
+                    }
+                    break;
                 }
                 ensureEnumerationSize(fEnumerationCount + 1);
                 fEnumeration[fEnumerationCount++] = token;
@@ -1319,8 +1328,11 @@ public class XMLDTDScannerImpl
      * [60] DefaultDecl ::= '#REQUIRED' | '#IMPLIED' | (('#FIXED' S)? AttValue)
      * </pre>
      *
-     * @param name The name of the attribute being scanned.
+     * @param elName
+     * @param atName The name of the attribute being scanned.
+     * @param type
      * @param defaultVal The string to fill in with the default value.
+     * @param nonNormalizedDefaultVal
      */
     protected final String scanAttDefaultDecl(String elName, String atName,
                                               String type,
@@ -2100,5 +2112,16 @@ public class XMLDTDScannerImpl
         // set starting state
         setScannerState(SCANNER_STATE_TEXT_DECL);
     }
+    
+    private int skipInvalidEnumerationValue() throws IOException {
+        int c;
+        do {
+            c = fEntityScanner.scanChar();
+        } 
+        while (c != '|' && c != ')');
+        ensureEnumerationSize(fEnumerationCount + 1);
+        fEnumeration[fEnumerationCount++] = XMLSymbols.EMPTY_STRING;
+        return c;
+    }
 
 } // class XMLDTDScannerImpl
