@@ -1,9 +1,10 @@
 /*
- * Copyright 2000-2006 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -73,7 +74,7 @@ import org.xml.sax.SAXException;
  * @author Pavani Mukthipudi, Sun Microsystems Inc.
  * @author Elena Litani, IBM
  * @author Rahul Srivastava, Sun Microsystems Inc.
- * @version $Id: DOMParserImpl.java 443059 2006-09-13 18:26:56Z mrglavas $
+ * @version $Id: DOMParserImpl.java 466933 2006-10-23 04:52:32Z mrglavas $
  */
 public class DOMParserImpl
     extends AbstractDOMParser implements LSParser, DOMConfiguration {
@@ -82,39 +83,42 @@ public class DOMParserImpl
 
     /** Feature identifier: namespaces. */
     protected static final String NAMESPACES =
-    Constants.SAX_FEATURE_PREFIX + Constants.NAMESPACES_FEATURE;
+        Constants.SAX_FEATURE_PREFIX + Constants.NAMESPACES_FEATURE;
 
     /** Feature id: validation. */
     protected static final String VALIDATION_FEATURE =
-    Constants.SAX_FEATURE_PREFIX+Constants.VALIDATION_FEATURE;
+        Constants.SAX_FEATURE_PREFIX+Constants.VALIDATION_FEATURE;
 
     /** XML Schema validation */
     protected static final String XMLSCHEMA =
-    Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_VALIDATION_FEATURE;
+        Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_VALIDATION_FEATURE;
     
     /** XML Schema full checking */
     protected static final String XMLSCHEMA_FULL_CHECKING =
-    Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_FULL_CHECKING;    
+        Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_FULL_CHECKING;    
 
     /** Dynamic validation */
     protected static final String DYNAMIC_VALIDATION =
-    Constants.XERCES_FEATURE_PREFIX + Constants.DYNAMIC_VALIDATION_FEATURE;
+        Constants.XERCES_FEATURE_PREFIX + Constants.DYNAMIC_VALIDATION_FEATURE;
 
     /** Feature identifier: expose schema normalized value */
     protected static final String NORMALIZE_DATA =
-    Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_NORMALIZED_VALUE;
+        Constants.XERCES_FEATURE_PREFIX + Constants.SCHEMA_NORMALIZED_VALUE;
 
     /** Feature identifier: disallow docType Decls. */
     protected static final String DISALLOW_DOCTYPE_DECL_FEATURE =
         Constants.XERCES_FEATURE_PREFIX + Constants.DISALLOW_DOCTYPE_DECL_FEATURE;
+    
+    /** Feature identifier: honour all schemaLocations */
+    protected static final String HONOUR_ALL_SCHEMALOCATIONS = 
+        Constants.XERCES_FEATURE_PREFIX + Constants.HONOUR_ALL_SCHEMALOCATIONS_FEATURE;
 
     // internal properties
     protected static final String SYMBOL_TABLE =
-    Constants.XERCES_PROPERTY_PREFIX + Constants.SYMBOL_TABLE_PROPERTY;
+        Constants.XERCES_PROPERTY_PREFIX + Constants.SYMBOL_TABLE_PROPERTY;
 
     protected static final String PSVI_AUGMENT =
-    Constants.XERCES_FEATURE_PREFIX +Constants.SCHEMA_AUGMENT_PSVI;
-
+        Constants.XERCES_FEATURE_PREFIX +Constants.SCHEMA_AUGMENT_PSVI;
 
     //
     // Data
@@ -339,8 +343,8 @@ public class DOMParserImpl
     public void setParameter (String name, Object value) throws DOMException {
         // set features
     	
-        if(value instanceof Boolean){
-            boolean state = ((Boolean)value).booleanValue ();
+        if (value instanceof Boolean) {
+            boolean state = ((Boolean)value).booleanValue();
             try {
                 if (name.equalsIgnoreCase (Constants.DOM_COMMENTS)) {
                     fConfiguration.setFeature (INCLUDE_COMMENTS_FEATURE, state);
@@ -438,7 +442,16 @@ public class DOMParserImpl
                     // Constants.DOM_CHARSET_OVERRIDES_XML_ENCODING feature,
                     // Constants.DOM_SPLIT_CDATA feature,
                     // or any Xerces feature
-                    fConfiguration.setFeature (name.toLowerCase(Locale.ENGLISH), state);
+                    String normalizedName;
+                    // The honour-all-schemaLocations feature is 
+                    // mixed case so requires special treatment.
+                    if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
+                        normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
+                    }
+                    else {
+                        normalizedName = name.toLowerCase(Locale.ENGLISH);
+                    }
+                    fConfiguration.setFeature(normalizedName, state);
                 }
 
             }
@@ -583,13 +596,38 @@ public class DOMParserImpl
                 fConfiguration.setProperty (DOCUMENT_CLASS_NAME, value);
             }
             else {
-                // REVISIT: check if this is a boolean parameter -- type mismatch should be thrown.
-                //parameter is not recognized
+                // Try to set the property.
+                String normalizedName = name.toLowerCase(Locale.ENGLISH);
+                try {
+                    fConfiguration.setProperty(normalizedName, value);
+                    return;
+                }
+                catch (XMLConfigurationException e) {}
+                
+                // If this is a boolean parameter a type mismatch should be thrown.
+                try {
+                    // The honour-all-schemaLocations feature is 
+                    // mixed case so requires special treatment.
+                    if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
+                        normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
+                    }
+                    fConfiguration.getFeature(normalizedName);
+                    String msg =
+                        DOMMessageFormatter.formatMessage (
+                                DOMMessageFormatter.DOM_DOMAIN,
+                                "TYPE_MISMATCH_ERR",
+                                new Object[] { name });
+                    throw new DOMException (DOMException.TYPE_MISMATCH_ERR, msg);
+                    
+                }
+                catch (XMLConfigurationException e) {}
+                
+                // Parameter is not recognized
                 String msg =
-                DOMMessageFormatter.formatMessage (
-                DOMMessageFormatter.DOM_DOMAIN,
-                "FEATURE_NOT_FOUND",
-                new Object[] { name });
+                    DOMMessageFormatter.formatMessage (
+                            DOMMessageFormatter.DOM_DOMAIN,
+                            "FEATURE_NOT_FOUND",
+                            new Object[] { name });
                 throw new DOMException (DOMException.NOT_FOUND_ERR, msg);
             }
         }
@@ -659,7 +697,7 @@ public class DOMParserImpl
                 ? Boolean.TRUE : Boolean.FALSE;
         }
         else if (name.equalsIgnoreCase(Constants.DOM_CHECK_CHAR_NORMALIZATION ) ||
-                 name.equalsIgnoreCase(Constants.DOM_NORMALIZE_CHARACTERS)){
+                 name.equalsIgnoreCase(Constants.DOM_NORMALIZE_CHARACTERS)) {
             return Boolean.FALSE;
         }
         else if (name.equalsIgnoreCase(Constants.DOM_NAMESPACE_DECLARATIONS)
@@ -684,12 +722,12 @@ public class DOMParserImpl
                 XMLEntityResolver entityResolver =
                 (XMLEntityResolver) fConfiguration.getProperty (ENTITY_RESOLVER);
                 if (entityResolver != null
-                && entityResolver instanceof DOMEntityResolverWrapper) {
-                    return ((DOMEntityResolverWrapper) entityResolver).getEntityResolver ();
+                        && entityResolver instanceof DOMEntityResolverWrapper) {
+                    return ((DOMEntityResolverWrapper) entityResolver).getEntityResolver();
                 }
-                return null;
             }
             catch (XMLConfigurationException e) {}
+            return null;
         }
         else if (name.equalsIgnoreCase (Constants.DOM_SCHEMA_TYPE)) {
             return fConfiguration.getProperty (
@@ -698,30 +736,52 @@ public class DOMParserImpl
         else if (name.equalsIgnoreCase (Constants.DOM_SCHEMA_LOCATION)) {
             return fSchemaLocation;
         }
-        else if (name.equalsIgnoreCase (SYMBOL_TABLE)){
+        else if (name.equalsIgnoreCase (SYMBOL_TABLE)) {
             return fConfiguration.getProperty (SYMBOL_TABLE);
         }
         else if (name.equalsIgnoreCase (DOCUMENT_CLASS_NAME)) {
             return fConfiguration.getProperty (DOCUMENT_CLASS_NAME);
         }
         else {
+            // This could be a recognized feature or property.
+            String normalizedName;
+            
+            // The honour-all-schemaLocations feature is 
+            // mixed case so requires special treatment.
+            if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
+                normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
+            }
+            else {
+                normalizedName = name.toLowerCase(Locale.ENGLISH);
+            }
+            try {
+                return fConfiguration.getFeature(normalizedName) 
+                    ? Boolean.TRUE : Boolean.FALSE;
+            }
+            catch (XMLConfigurationException e) {}
+            
+            // This isn't a feature; perhaps it's a property
+            try {
+                return fConfiguration.getProperty(normalizedName);
+            }
+            catch (XMLConfigurationException e) {}
+            
             String msg =
-            DOMMessageFormatter.formatMessage (
-            DOMMessageFormatter.DOM_DOMAIN,
-            "FEATURE_NOT_FOUND",
-            new Object[] { name });
+                DOMMessageFormatter.formatMessage (
+                        DOMMessageFormatter.DOM_DOMAIN,
+                        "FEATURE_NOT_FOUND",
+                        new Object[] { name });
             throw new DOMException (DOMException.NOT_FOUND_ERR, msg);
         }
-        return null;
     }
 
     public boolean canSetParameter (String name, Object value) {
-    	if (value == null){
-    		return true;
-    	}
-    	
-        if(value instanceof Boolean){
-            boolean state = ((Boolean)value).booleanValue ();
+        if (value == null) {
+            return true;
+        }
+        
+        if (value instanceof Boolean) {
+            boolean state = ((Boolean)value).booleanValue();
             if ( name.equalsIgnoreCase (Constants.DOM_SUPPORTED_MEDIATYPES_ONLY)
             || name.equalsIgnoreCase(Constants.DOM_NORMALIZE_CHARACTERS)
             || name.equalsIgnoreCase(Constants.DOM_CHECK_CHAR_NORMALIZATION )
@@ -752,7 +812,16 @@ public class DOMParserImpl
 
             // Recognize Xerces features.
             try {
-                fConfiguration.getFeature(name.toLowerCase(Locale.ENGLISH));
+                String normalizedName;
+                // The honour-all-schemaLocations feature is 
+                // mixed case so requires special treatment.
+                if (name.equalsIgnoreCase(HONOUR_ALL_SCHEMALOCATIONS)) {
+                    normalizedName = HONOUR_ALL_SCHEMALOCATIONS;
+                }
+                else {
+                    normalizedName = name.toLowerCase(Locale.ENGLISH);
+                }
+                fConfiguration.getFeature(normalizedName);
                 return true;
             }
             catch (XMLConfigurationException e) {
@@ -785,10 +854,18 @@ public class DOMParserImpl
                     return true;
                 return false;
             }
-            else if (name.equalsIgnoreCase (DOCUMENT_CLASS_NAME)){
+            else if (name.equalsIgnoreCase (DOCUMENT_CLASS_NAME)) {
                 return true;
             }
-            return false;
+            
+            // Recognize Xerces properties.
+            try {
+                fConfiguration.getProperty(name.toLowerCase(Locale.ENGLISH));
+                return true;
+            }
+            catch (XMLConfigurationException e) {
+                return false;
+            }
         }
     }
 
@@ -1053,7 +1130,7 @@ public class DOMParserImpl
     }
 
     /**
-     * @see org.w3c.dom.ls.DOMParser#abort()
+     * @see org.w3c.dom.ls.LSParser#abort()
      */
     public void abort () {
         // If parse operation is in progress then reset it
